@@ -3,8 +3,9 @@
 
 /* ANTLR Translator Generator
  * Project led by Terence Parr at http://www.jGuru.com
- * Software rights: http://www.antlr.org/RIGHTS.html
+ * Software rights: http://www.antlr.org/license.html
  *
+ * $Id: CharScanner.hpp 669650 2007-05-29 22:44:14Z mueller $
  */
 
 #include <antlr/config.hpp>
@@ -16,9 +17,17 @@
 #else
 #include <cctype>
 #endif
+#include <cstring>
+
+#if ( _MSC_VER == 1200 )
+// VC6 seems to need this
+// note that this is not a standard C++ include file.
+# include <stdio.h>
+#endif
 
 #include <antlr/TokenStream.hpp>
 #include <antlr/RecognitionException.hpp>
+#include <antlr/SemanticException.hpp>
 #include <antlr/MismatchedCharException.hpp>
 #include <antlr/InputBuffer.hpp>
 #include <antlr/BitSet.hpp>
@@ -61,7 +70,7 @@ private:
 	const CharScanner* scanner;
 public:
 #ifdef NO_TEMPLATE_PARTS
-	CharScannerLiteralsLess(); // not really used
+	CharScannerLiteralsLess() {} // not really used, definition to appease MSVC
 #endif
 	CharScannerLiteralsLess(const CharScanner* theScanner)
 	: scanner(theScanner)
@@ -87,22 +96,25 @@ public:
 	{
 	}
 
-	virtual int LA(int i);
+	virtual int LA(unsigned int i);
 
 	virtual void append(char c)
 	{
-		if (saveConsumedInput) {
-			int l = text.length();
+		if (saveConsumedInput)
+		{
+			size_t l = text.length();
+
 			if ((l%256) == 0)
 				text.reserve(l+256);
+
 			text.replace(l,0,&c,1);
 		}
 	}
 
 	virtual void append(const ANTLR_USE_NAMESPACE(std)string& s)
 	{
-		if (saveConsumedInput)
-			text+=s;
+		if( saveConsumedInput )
+			text += s;
 	}
 
 	virtual void commit()
@@ -110,7 +122,39 @@ public:
 		inputState->getInput().commit();
 	}
 
-	virtual void consume();
+	/** called by the generated lexer to do error recovery, override to
+	 * customize the behaviour.
+	 */
+	virtual void recover(const RecognitionException& ex, const BitSet& tokenSet)
+	{
+		consume();
+		consumeUntil(tokenSet);
+	}
+
+	virtual void consume()
+	{
+		if (inputState->guessing == 0)
+		{
+			int c = LA(1);
+			if (caseSensitive)
+			{
+				append(c);
+			}
+			else
+			{
+				// use input.LA(), not LA(), to get original case
+				// CharScanner.LA() would toLower it.
+				append(inputState->getInput().LA(1));
+			}
+
+			// RK: in a sense I don't like this automatic handling.
+			if (c == '\t')
+				tab();
+			else
+				inputState->column++;
+		}
+		inputState->getInput().consume();
+	}
 
 	/** Consume chars until one matches the given char */
 	virtual void consumeUntil(int c)
@@ -137,12 +181,12 @@ public:
 	}
 
 	/// Mark the current position and return a id for it
-	virtual int mark()
+	virtual unsigned int mark()
 	{
 		return inputState->getInput().mark();
 	}
 	/// Rewind the scanner to a previously marked position
-	virtual void rewind(int pos)
+	virtual void rewind(unsigned int pos)
 	{
 		inputState->getInput().rewind(pos);
 	}
@@ -161,23 +205,43 @@ public:
 	 */
 	virtual void match(const BitSet& b)
 	{
-		if (!b.member(LA(1))) {
-			throw MismatchedCharException(LA(1),b,false,this);
-		}
+		int la_1 = LA(1);
+
+		if ( !b.member(la_1) )
+			throw MismatchedCharException( la_1, b, false, this );
 		consume();
 	}
 
-	/// See if input contains string 's' throw MismatchedCharException if not
+	/** See if input contains string 's' throw MismatchedCharException if not
+	 * @note the string cannot match EOF
+	 */
+	virtual void match( const char* s )
+	{
+		while( *s != '\0' )
+		{
+			// the & 0xFF is here to prevent sign extension lateron
+			int la_1 = LA(1), c = (*s++ & 0xFF);
+
+			if ( la_1 != c )
+				throw MismatchedCharException(la_1, c, false, this);
+
+			consume();
+		}
+	}
+	/** See if input contains string 's' throw MismatchedCharException if not
+	 * @note the string cannot match EOF
+	 */
 	virtual void match(const ANTLR_USE_NAMESPACE(std)string& s)
 	{
-		int len = s.length();
+		size_t len = s.length();
 
-		for (int i = 0; i < len; i++)
+		for (size_t i = 0; i < len; i++)
 		{
-			int la_1 = LA(1);
+			// the & 0xFF is here to prevent sign extension lateron
+			int la_1 = LA(1), c = (s[i] & 0xFF);
 
-			if ( la_1 != s[i] )
-				throw MismatchedCharException(la_1, s[i], false, this);
+			if ( la_1 != c )
+				throw MismatchedCharException(la_1, c, false, this);
 
 			consume();
 		}
@@ -287,7 +351,7 @@ public:
 	}
 
 	/** Used to keep track of line breaks, needs to be called from
-	 * within generated lexers when a "\n \r" is encountered.
+	 * within generated lexers when a \n \r is encountered.
 	 */
 	virtual void newline()
 	{
@@ -296,7 +360,8 @@ public:
 	}
 
 	/** Advance the current column number by an appropriate amount according
-	 * to the tabsize. This methad is called automatically from consume()
+	 * to the tabsize. This method needs to be explicitly called from the
+	 * lexer rules encountering tabs.
 	 */
 	virtual void tab()
 	{
@@ -317,11 +382,6 @@ public:
 		return tabsize;
 	}
 
-	/// Called when a unrecoverable error is encountered
-	void panic();
-	/// Called when a unrecoverable error is encountered
-	void panic(const ANTLR_USE_NAMESPACE(std)string& s);
-
 	/** Report exception errors caught in nextToken() */
 	virtual void reportError(const RecognitionException& e);
 
@@ -484,7 +544,7 @@ private:
 #endif
 };
 
-inline int CharScanner::LA(int i)
+inline int CharScanner::LA(unsigned int i)
 {
 	int c = inputState->getInput().LA(i);
 
